function y=SMIC(x,c)

% SMI-based Clustering
%
% y=SMIC(x,c)
%    x: data matrix (d x n, d:dimensionality, n:the number of samples)
%    c: the number of clusters (positive integer)
%    y: cluster assignment (1 x n)
%
% (c) Masashi Sugiyama, Department of Compter Science, Tokyo Institute of Technology, Japan.
%     sugi@cs.titech.ac.jp,     http://sugiyama-www.cs.titech.ac.jp/~sugi

if nargin<2
  c=2;
end

[d,n]=size(x);
tmp=repmat(sum(x.^2,1),[n 1]);
dist2=tmp+tmp'-2*x'*x;
kparam_list=[1:10];
OPTS.disp=0;

SMIh=-inf;
for kparam_index=1:length(kparam_list)
  kparam=kparam_list(kparam_index);

  % Compute kernel matrix (sparse local scaing)
  [sorted,index] = sort(dist2);
  sigma=sqrt(sorted(kparam+1,:));
  localscale=sigma'*sigma;
  flag1=(localscale~=0);
  flag2=sparse(dist2<=repmat(sorted(kparam+1,:),[size(dist2,1) 1]));
  flag=sparse(flag1&(flag2|flag2'));
  K=sparse(zeros(size(dist2,1)));
  K(flag)=exp(-dist2(flag)./localscale(flag));
  K=(K+K')/2;

  % Perform Clustering
  [Phi,Lambda]=eigs(K,c,'LM',OPTS);
  oPhi=max(0,Phi.*repmat(sign(sum(Phi)),[n 1]));
  P=oPhi./repmat(sum(oPhi),[n 1]);
  [dummy y0]=max(P,[],2);

  % Compute SMI score
  SMIh0=LSMIclassification(x,y0');
  disp(sprintf('kernel parameter=%g,  SMIh=%g',kparam,SMIh0));
  if SMIh0>SMIh
    y=y0';
    SMIh=SMIh0;
  end
end
