% Demo script of SMI-based clustering
%
% (c) Masashi Sugiyama, Department of Compter Science, Tokyo Institute of Technology, Japan.
%     sugi@cs.titech.ac.jp,     http://sugiyama-www.cs.titech.ac.jp/~sugi

clear all

rand('state',1);
randn('state',1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Generate Data
dataset=1;
disp(sprintf('dataset=%g',dataset));
switch dataset
 case 1 % Two Gaussians
  n=200; c=2; d=2;
  flag=sort(rand(1,n)>0.5);
  x=randn(2,n)+5*[(flag)*2-1; zeros(1,n)];
  ytrue=flag+1;
 case 2 % Four Gaussians
  n=200; c=4; d=2;
  flag1=(rand(1,n)>0.5); flag2=(rand(1,n)>0.5);
  x=0.5*randn(2,n)+[flag1*4-2; flag2*4-2];
  ytrue=flag1+flag2*2+1;
 case 3 % Spiral
  n=200; c=2; d=2;
  x=zeros(2,n);
  for i=1:n/2
    r=1+4*(i-1)/n;  t=(i-1)*3/n*pi;
    x(1,i)=r*cos(t); x(2,i)=r*sin(t);
    x(1,i+n/2)=r*cos(t+pi); x(2,i+n/2)=r*sin(t+pi);
  end
  x=x+0.1*randn(2,n);
  ytrue=[ones(1,n/2) 2*ones(1,n/2)];
 case 4 % High and low densities
  n=200; c=2; d=2;
  x=[0.1*randn(2,n/2) randn(2,n/2)];
  ytrue=[ones(1,n/2) 2*ones(1,n/2)];
 case 5 % Circle and Gaussian
  n=200; c=2; d=2;
  x(1,:)=[5*cos(linspace(0,2*pi,n/2)) randn(1,n/2)];
  x(2,:)=[5*sin(linspace(0,2*pi,n/2)) randn(1,n/2)];
  x=x+0.1*randn(2,n);
  ytrue=[ones(1,n/2) 2*ones(1,n/2)];
end

x=x-repmat(mean(x,2),[1 n]);   % Centralization
x=x./repmat(std(x,1,2),[1 n]); % Normalization

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Perform Clustering

y=SMIC(x,c);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Display Results

SYMBOL={'bo','rx','g*','ks'};
figure(1); clf; hold on
set(gca,'FontName','Helvetica','FontSize',12)
for yy=1:c
  plot(x(1,y==yy),x(2,y==yy),SYMBOL{yy});
end
set(gcf,'PaperUnits','centimeters','PaperPosition',[0 0 9 9]);

%print('-depsc',sprintf('data%g',dataset))
print('-dpng',sprintf('data%g',dataset))

